"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Jobs = void 0;
const axios_1 = require("axios");
const auth_1 = require("./util/auth");
const scopes = ['https://www.googleapis.com/auth/cloud-platform'];
const JOBS_API_HOST = "https://us-central1-run.googleapis.com/apis/run.googleapis.com/v1/namespaces/";
async function getAxiosConfig(projectName, serviceAccount) {
    const headers = await (0, auth_1.getAuthHeader)(projectName, serviceAccount, scopes);
    return { headers };
}
class Jobs {
    constructor(projectName, serviceAccount) {
        this.projectName = projectName;
        this.serviceAccount = serviceAccount;
    }
    async createJob(createJobArgs) {
        const { jobName, image, command = [], args = [], env = [], timeoutSeconds = 600, maxRetries = 3, cpu = "1000m", memory = "512Mi" } = createJobArgs;
        const config = await getAxiosConfig(this.projectName, this.serviceAccount);
        config.headers["Content-Type"] = 'application/json';
        const body = {
            apiVersion: "run.googleapis.com/v1",
            kind: "Job",
            metadata: {
                name: jobName,
                namespace: this.serviceAccount.split('-')[0],
                generation: 2
            },
            spec: {
                template: {
                    spec: {
                        taskCount: 1,
                        template: {
                            spec: {
                                containers: [
                                    {
                                        image,
                                        command,
                                        args,
                                        env,
                                        resources: {
                                            limits: {
                                                cpu,
                                                memory
                                            }
                                        },
                                    }
                                ],
                                timeoutSeconds: timeoutSeconds.toString(),
                                serviceAccountName: this.serviceAccount,
                                maxRetries
                            }
                        }
                    }
                }
            }
        };
        const axios = new axios_1.Axios({});
        const { data } = await axios.post(JOBS_API_HOST + this.projectName + '/jobs', JSON.stringify(body), config);
        const parsedData = JSON.parse(data);
        if (parsedData.error)
            throw Error(JSON.stringify(parsedData.error));
        return parsedData;
    }
    async isJobRunning(jobId) {
        const config = await getAxiosConfig(this.projectName, this.serviceAccount);
        config.params = {
            page_size: 1,
            labelSelector: `run.googleapis.com/job=${jobId}`
        };
        const axios = new axios_1.Axios({});
        try {
            const response = await axios.get(JOBS_API_HOST + this.projectName + '/executions', config);
            const data = JSON.parse(response.data);
            if (data.items?.length > 0) {
                const runningCount = data.items[0]?.status?.runningCount || 0;
                return runningCount > 0;
            }
            else
                return false;
        }
        catch (e) {
            throw Error(e.message);
        }
    }
    async runJob(jobId) {
        try {
            const config = await getAxiosConfig(this.projectName, this.serviceAccount);
            const axios = new axios_1.Axios({});
            const response = await axios.post(JOBS_API_HOST + this.projectName + '/jobs/' + jobId + ':run', "", config);
            return response.status == 200;
        }
        catch (e) {
            return false;
        }
    }
    async listJobs() {
        const config = await getAxiosConfig(this.projectName, this.serviceAccount);
        const axios = new axios_1.Axios({});
        const { data } = await axios.get(JOBS_API_HOST + this.projectName + '/jobs', config);
        return JSON.parse(data);
    }
    async getJob(jobId) {
        const config = await getAxiosConfig(this.projectName, this.serviceAccount);
        const axios = new axios_1.Axios({});
        const { data } = await axios.get(JOBS_API_HOST + this.projectName + '/jobs/' + jobId, config);
        const parsedData = JSON.parse(data);
        if (parsedData.error)
            throw Error(JSON.stringify(parsedData.error));
        return parsedData;
    }
    async deleteJob(jobId) {
        const config = await getAxiosConfig(this.projectName, this.serviceAccount);
        const axios = new axios_1.Axios({});
        const axiosResponse = await axios.delete(JOBS_API_HOST + this.projectName + '/jobs/' + jobId, config);
        const parsedData = JSON.parse(axiosResponse.data);
        if (parsedData.error)
            throw Error(JSON.stringify(parsedData.error));
        return parsedData;
    }
    async listExecutions(jobId) {
        const config = await getAxiosConfig(this.projectName, this.serviceAccount);
        config.params = {
            labelSelector: `run.googleapis.com/job=${jobId}`
        };
        const axios = new axios_1.Axios({});
        const { data } = await axios.get(JOBS_API_HOST + this.projectName + '/executions', config);
        return JSON.parse(data);
    }
    async getJobExecutionHealth(jobId, take) {
        if (take <= 0)
            throw new Error("take must be positive");
        const job = await this.getJob(jobId);
        const listExecutions = await this.listExecutions(jobId);
        const items = listExecutions.items || [];
        return {
            executionCount: job.status?.executionCount || 0,
            lastFailedCount: items?.slice(0, take).filter(execution => (execution?.status?.failedCount || 0) > 0).length || 0,
            lastExecutionStatus: items[0]?.status?.conditions?.find(condition => condition.type == "Completed")?.status || "Unknown",
            startTime: items[0]?.status?.startTime,
            completionTime: items[0]?.status?.completionTime
        };
    }
}
exports.Jobs = Jobs;
